import { dataSource } from "../../../data-source";
import { protocolDecision } from "../../../entity/club/protocol/protocolDecision";
import DatabaseActionException from "../../../exceptions/databaseActionException";
import InternalException from "../../../exceptions/internalException";
import { SynchronizeProtocolDecisionCommand } from "./protocolDecisionCommand";

export default abstract class ProtocolDecisionCommandHandler {
  /**
   * @description create protocolDecision
   * @param {number} protocolId
   * @returns {Promise<number>}
   */
  static async create(protocolId: number): Promise<number> {
    return await dataSource
      .createQueryBuilder()
      .insert()
      .into(protocolDecision)
      .values({
        topic: "",
        context: "",
        protocolId,
      })
      .execute()
      .then((result) => {
        return result.identifiers[0].id;
      })
      .catch((err) => {
        throw new DatabaseActionException("CREATE", "protocolDecision", err);
      });
  }
  /**
   * @description sync protocolDecision
   * @param {Array<SynchronizeProtocolDecisionCommand>} syncProtocolDecisions
   * @returns {Promise<void>}
   */
  static async sync(syncProtocolDecisions: Array<SynchronizeProtocolDecisionCommand>): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .insert()
      .into(protocolDecision)
      .values(syncProtocolDecisions)
      .orUpdate(["topic", "context"], ["id"])
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("SYNC", "protocolDecision", err);
      });
  }
}
